;**************************************************************************
; RPOT001.ASM
;
; PROGRAM: 	Demonstrator to show how to read POT position and scale result 
;		to reduce effects of component tolerence.
;
; DESCRIPTION: 	On a trigger signal start a 0 to 5 min timer.
;		This program uses an RC timer to set the time.
; 		The Pot needs to be calibrated, and code does a 
;		self calibrate.
;
;	Circuit 2: Mininal Component version
;
;		PA3 ---[ 100 k pot ]--- PA2
;			 A   ^   B
;                            |
;			     +--------- PA4
;                            |
;			     |		    | |
;			     +--[ 220R ]----| |------ gnd
;					    | |
;	                                1uF capacitor
;
;		The design also uses an RS232 debug output, all in Hex
;
;		The format is: 
;		t:A=(A+B):meas<Range:output
;		t:72=95:72<95:00E6
;		t:72=95:72<95:00E6
;		t:72=95:72<95:
;		t:8B=B6:8B<B6:00E6
;		t:8B=B6:8B<B6:00E6
;		t:8B=B6:8B<B6:00E6
;		
;
;
;		RC is measured by applying a Logic 1 to PA3 or PA2.
;		Count up RCcounter until PA4 gets to logic 1
;		When the voltage gets to logic one on PA4 sample count
;
;		Rmin,Rnow,Rmax are counts. 
;
;		Convert the ratio of (Rnow-Rmin)/(Rmax-Rmin) to 0 to 5 min duration
;		using a successive approximation.
;		
;		Duration is in seconds.
;		Countdown-0x0101= seconds to go
;		* - timer running

; AUTHOR:  	Douglas Rice
; Copyright 2001
;
;	Crystal:	32kHz
;
;	I/O used:-
;	PORTA:0	RS232 debug for Manufacture / Design Test 
;	PORTA:1 not used 
;	PORTA:2 RC measurement B 
;	PORTA:3 RC measurement A
;	PORTA:4	RC discharge
;
;	
;	PORTB:2	Output C - Red    LED pulled low  to illuminate
;	PORTB:1	Output B - Yellow LED pulled high to illuminate
;	PORTB:0	Output A - Green  LED pulled low  to illuminate
;
;
;
	INCLUDE "p16F84.inc"       
;
;**************************************************************************

        LIST P=16F84, R=DEC
	__idlocs  0x1234
	__config  _XT_OSC&_PWRTE_ON & _WDT_OFF

	cblock 	0x0c
		; variables used for context saving 
		w_temp
		status_temp
	endc

;--------------------------------------------------------------------------
; Sec 0. Build Constants - uncomment to include in the code.
;--------------------------------------------------------------------------
;

buildDEBUG	; if defined code easier to debug when single stepping.
buildCircuit2
buildMacroRS232
;--------------------------------------------------------------------------
; Sec 1. Equates and Constants
;--------------------------------------------------------------------------

		; The General Purpose Registers start at the end of the 
		; Special Purpose Registers.

;--------------------------------------------------------------------------
; Sec 1.2 Button and LED 
;--------------------------------------------------------------------------

; PORTA binary values
; some port A bits are set in AWAI.inc

PORT_RS232	EQU	PORTA
RS232OUT	EQU	0

;--------------------------------------------------------------------------
; Sec 1.3 Table Macro
;--------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary near " START   " "
	endif
	endm

;--------------------------------------------------------------------------
; Sec 2.0 Variables
;--------------------------------------------------------------------------
; Variables start 0x0C
;
	cblock 	
SRout 		; for RS232 routines

SRin	; Used in an interupt routine for serial Input
SRcnt		
SRloopCnt

PORTBtemp

	;
	ENDC
	ifdef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTBtemp
	endif
	ifndef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTB
	endif
;--------------------------------------------------------------------------
; Sec 3. Program Code
;--------------------------------------------------------------------------

	ORG     0       
        GOTO    Start

	ORG 	4
	GOTO 	Intrtn

;--------------------------------------------------------------------------
; Sec 3.1	Main Program Init Code
;--------------------------------------------------------------------------

Start
	; Change Prescaler
	CLRWDT
	; Configure Tmr 0
	BSF	STATUS,RP0
	; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
	MOVLW	1 << INTEDG | 0x0 + 2 	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16
	MOVWF	OPTION_REG
	BCF	STATUS,RP0


	CALL	FLled1		; debug 
	CALL	RCmeasureR2
	CALL	FLled2		; debug 
	CALL	MRcalcABdivC_A
	CALL	FLled3		; debug 
	CALL	SRlistMeasurements
	goto Start

;--------------------------------------------------------------------------
; Sec 3.2	Main Program 
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Sec 4.10	RC measurment routine
;--------------------------------------------------------------------------
;				

;    GND --| |--[ 200R ]--+---[ R2 ]---- RA2
;	    C 	          |
;		          +---[ R3 ]---- RA3
;		          |
;		          +---RA4/TOCLKI
;
; R2 and R3 form the two sides of a pot.
;	---[ R2    R3 ]---
;                ^
;                |
;                +--------
;
; To measure CR enable either RA2 or RA3  as an output and RA4 as an input. 
;
; Set RA2 or RA3 to start charging C, use the input to see if it has got to logic 1.
;

	cblock
RCresultR2
RCresultR3
	endc

RCmeasureR2
	
	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA
	CLRF	RCresultR2
	MOVLW	~( 1 << 3 )
	; count how long it takes to charge C
	CALL	RCmeas2
	MOVWF	RCresultR3
	; Make RA4 an output and zero it to discharge C
	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	;	leave RCresultR2 then measure R3
	CLRF	PORTA

	; Make RA3 an output and set it to charge C
	; put the result of A into RCresultR3
	; Keep adding to RCresultR2 to measure R3+R2
	; so do not use:
	;  CLRF	RCresultR2
	MOVLW	~( 1 << 2 )
	CALL	RCmeas2

	MOVLW	~( 1 << 4 )
	TRIS	PORTA
	CLRF	PORTA		
	RETURN

RCmeas2
	TRIS	PORTA
	MOVLW	0xFF
	MOVWF	PORTA		; Set output high to start Measurement
RCmeasR2Loop	
	BTFSC	PORTA,4
	GOTO	RCmeasureR2End
	INCFSZ	RCresultR2
	GOTO 	RCmeasR2Loop
RCmeasureR2End
	DECFSZ	RCresultR2 	; limit max
	MOVFW	RCresultR2
	RETURN




	; You could leave RCresultR2 anduse its value
	; as the starting count for RCresultR2
	; to get the sum or R2 + R3

;MMnextTry    := VinRef; 
;MMoutNextTry := VoutRef; 
;MMitteration := 0;
;MMoutput     := 0;
;
;For MMprecision  := 0 to 8 do
;BEGIN
;  MMnextTry    := MMnextTry div 2;
;  MMoutNextTry := MMoutNextTry div 2;
;
;  ; Compare Vmeas against itteration
;
;  if ( MMmeas => ( MMitteration+MMnextTry )
;  BEGIN
;    MMitteration := MMitteration + MMnextTry;
;    MMoutput     := MMoutput + VoutNextTry
;  END
;END

	cblock
MMmeasA
MMitteration
MMnextTry
MMoutNextTry
MMoutputhA
MMoutputlA
MMprecision
	endc
		
MRcalcABdivC_A
	MOVFW	RCresultR3	; use R2 as measured vaule
	MOVWF	MMmeasA
	CLRC			; Add R2+R3 to give range
	MOVFW	RCresultR2
	; no need to add R3  to R2 as R2 already if R3+R2
;	ADDWF	RCresultR3,w
	MOVWF	MMnextTry

MRcalcABdivC2
      
	; set to 150, not 300 to save 16 arithmatic
	MOVLW	.150
	MOVWF	MMoutNextTry	; this is divided by 2

	CLRF	MMitteration
	CLRF	MMoutputhA
	CLRF	MMoutputlA
	movlw	8
	movwf	MMprecision
MMcalcLoop2	
	clrc
	rrf	MMnextTry,f
	;	MMrangeT needs to be 300 seconds which is > 255 so divide by 2 now
	; calc  if MMmeas > MMitteration+MMnextTry/2	
	clrc
	movfw	MMnextTry
	ADDWF	MMitteration,w
	SUBWF	MMmeasA,w	; w= f-w
	BNC	MMcalc2 
	BZ	MMcalc2 

	; work out new nextTry for meas
	movfw	MMnextTry
	addwf	MMitteration,f
	; Slave Calc the Timer value
	movfw	MMoutNextTry
	addwf	MMoutputlA,f
	BNC	MMcalc2 
	incfsz	MMoutputhA,f
MMcalc2
	clrc
	rrf	MMoutNextTry,f
	decfsz	MMprecision,f
	goto MMcalcLoop2

	; MMoutputlA and MMoutputhA

	RETURN



;--------------------------------------------------------------------------
; Sec 4.9	Flash LEDs
;--------------------------------------------------------------------------
FLled1
	BSF	STATUS, RP0
	MOVLW	~7
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	~1
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN

FLled2
	BSF	STATUS, RP0
	MOVLW	~7
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	~4
	MOVWF	PORTB		; Set output high to start Measurement


FLled3
	BSF	STATUS, RP0
	MOVLW	~7
	MOVWF	TRISB		; Set output high to start Measurement
	BCF	STATUS, RP0
	MOVLW	~2
	MOVWF	PORTB		; Set output high to start Measurement
	RETURN
		
;--------------------------------------------------------------------------
; Sec 4.9	Serial Out - Generic routine
;--------------------------------------------------------------------------
;				
;		@ 8192 instructions a second, 
;		for 1200 baud, each bit = 6.82666 instructions
;		
;		---_01234567P---- 
;		-> Start     Stop
;		The output signal is inverted by the driver transistor
;
;		8192/	instrns	cumulative	% er
;		1200
;
;		0.000		0	
;	start	6.827	7	7	102.54%
;	0	13.653	7	14	102.54%
;	1	20.480	7	21	102.54%
;	2	27.307	6	27	98.88%
;	3	34.133	7	34	99.61%
;	4	40.960	7	41	100.10%
;	5	47.787	7	48	100.45%
;	6	54.613	7	55	100.71%
;	7	61.440	6	61	99.28%
;	8	68.267	7	68	99.61%
;	end	75.093	7	75	99.88%
;					

;#Is the output inverted? 
;# Normally the UART OP is inverted by the Line driver
;# but as design does not uses a driver invert the bits
RS232opHigh	MACRO	
		BCF	PORT_RS232,RS232OUT
	endm

RS232opLow	MACRO	
		BSF	PORT_RS232,RS232OUT
	endm

RS232opBit	MACRO   bit
		BTFSC	SRout,bit		; 1 if false, 2 if skip
		GOTO	SRtxCharA#v(bit)	; 	t+1 instructions
		NOP			;t+2
		;BSF	PORT_RS232,RS232OUT		;t+3
		RS232opLow
		GOTO	SRtxCharA#v(bit)a	;t+4
SRtxCharA#v(bit)	;BCF	PORT_RS232,RS232OUT		;	t+3
		RS232opHigh

		NOP			;	t+4
		NOP			;	t+5
SRtxCharA#v(bit)a	NOP			;t+6	t+6
	endm

	ifdef buildMacroRS232
SRtxChar
		;BSF	PORT_RS232,RS232OUT		;		t=0
		RS232opLow
;		MOVLW	'U'
		MOVWF	SRout

		NOP			;		t+1
		NOP			;		t+2		
;		NOP			;		t+3
		; These take 4 instruction cyles
		; t=0
		NOP


		RS232opBit	0
		RS232opBit	1
		RS232opBit	2
		RS232opBit	3
		RS232opBit	4
		RS232opBit	5
		RS232opBit	6
		RS232opBit	7
		; Stop Bit
		RS232opHigh
;		BCF	PORT_RS232,RS232OUT

		NOP
;		NOP
;		NOP
;		NOP
;		NOP
;		NOP
		RETURN
	endif

SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar

;--------------------------------------------------------------------------
; Sec 4.9	Serial Out
;--------------------------------------------------------------------------


SRlistMeasurements

	MOVLW	~( 1 << 0 )
;	TRIS	PORTA
	BSF	STATUS, RP0
	MOVWF	TRISA		; Set output high to start Measurement
	BCF	STATUS, RP0


	CLRF	PORTA		

	MOVLW	0x0D
	CALL	SRtxChar	
	MOVLW	0x0A
	CALL	SRtxChar	
	MOVLW	't'
	CALL	SRtxChar	
	MOVLW	':'
	CALL	SRtxChar	
	
	SWAPF	RCresultR3,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR3
	CALL	SRwrtHexNibble	

	MOVLW	'='
	CALL	SRtxChar	

	SWAPF	RCresultR2,w
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR2
	CALL	SRwrtHexNibble	

	MOVLW	':'
	CALL	SRtxChar	

	SWAPF	RCresultR3,W
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR3
	CALL	SRwrtHexNibble	

	MOVLW	'<'
	CALL	SRtxChar	


	SWAPF	RCresultR2,W
	CALL	SRwrtHexNibble	
	MOVFW	RCresultR2
	CALL	SRwrtHexNibble	

	MOVLW	':'

	CALL	SRtxChar	
	SWAPF	MMoutputhA,w
	CALL	SRwrtHexNibble	
	MOVFW	MMoutputhA
	CALL	SRwrtHexNibble	

	SWAPF	MMoutputlA,w
	CALL	SRwrtHexNibble	
	MOVFW	MMoutputlA
	CALL	SRwrtHexNibble	
	RETURN
		

;--------------------------------------------------------------------------
; Sec 5.0	Interrupt Routines
;--------------------------------------------------------------------------

Intrtn	

		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		; Which Interupt ?
		; No Context Save in these routines.

IntrtnEnding
		BCF 	INTCON,INTF
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt


CodeEnd		goto	CodeEnd

;--------------------------------------------------------------------------
; Program End
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; EEPROM data
;--------------------------------------------------------------------------
	ORG	0x2100
	cblock
LASTcont
	endc
        END


t:72=95:72<95:00E6
t:72=95:72<95:00E6
t:72=95:72<95:00E6
t:72=95:72<95:00E6
t:71=94:71<94:00E4
t:71=94:71<94:00E4
t:71=94:71<94:00E4
t:72=95:72<95:
t:89=B4:89<B4:00E4
t:8A=B5:8A<B5:00E6
t:7E=A9:7E<A9:00DD
t:8C=B7:8C<B7:00E8
t:8B=B6:8B<B6:00E6
t:8C=B7:8C<B7:00E8
t:8B=B6:8B<B6:00E6
t:8B=B6:8B<B6:00E6
t:8B=B6:8B<B6:00E6
t:8B=B6:8B<B6:00E6
t:8B=B6:8B<B6:00E6
t:8B=B6:8B<B6:00E6
t:8B=B6:8Bn
t:72=95:72<95:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:73=
t:6E=91:6E<91:00E4
t:6D=90:6D<90:00E2
t:72=94:72<94:00E6
t:71=93:71<93:00E8
t:71=94:71<94:00E4
t:72=94:72<94:00E6
t:73=95:73<95:00E8
t:71=93:71<93:00E8
t:72=94:72<94:00E6
t:72=94:72<94:00E6
t:72=94:72<94:00E6
t:72=94:72<94:00E6
t:74=97:74<97:00E8
t:73=96:73<96:00E6
t:73=96:73<96:00E6
t:72=95:72<95:00E6
t:72=95:72<95:00E6
t:70=92:70<92:00E6
t:68=8C:68<8C:00DD
t:73=96:73<96:00E6
t:71=93:71<93:00E8
t:72=94:72<94:00E6
t:72=94:72<94:00E6
t:73=95:73<95:00E8
t:73=96:73<96:00E6
