;**************************************************************************
; Video Timer.ASM
;
; PROGRAM: 	Timer to start and stop a video. 
;
; DESCRIPTION: 	On a trigger signal start a 0 to 5 min timer.
;		This program uses an RC timer to set the time.
; 		The Pot needs to be calibrated, and code does a 
;		self calibrate.
;
;	Circuit 1:
;
;		The self calibrate works when the Chip is reset.
;		On chip reset the pot must be set to min.
;
;		PA3 ---[ 1k ]---[ 100 k pot ]---[ 1k ]--- PA2
;			  A	      ^		   B
;                                     |
;				      +--[ 220R ]--- to RA4
;				      |
;				    -----
;                                   -----  1uF capacitor
;                                     |
;                                     | gnd
;
; 	
;		The self calibrate works when the Chip is reset.
;		On chip reset the pot must be set to min.
;
;	Circuit 2: Mininal Component version
;
;		PA3 ---[ 100 k pot ]--- PA2
;			 A   ^   B
;                            |
;			     +--------- PA4
;                            |
;			     |		    | |
;			     +--[ 220R ]----| |------ gnd
;					    | |
;	                                1uF capacitor
;
;		The design also uses an RS232 debug output, all in Hex
;
;		The format is: 
;		t:meas:Rmin>Rcurrent>Rmax:Duration:Countdown:*
;		tmeas:A+B=Sum:min<C<max:time,count down: 
;		tA1:A9+34=DD:08<A9<D8:00E8,0000:
;		tA1:A9+34=DD:08<A9<D8:00E8,0000:
;		tA1:A9+34=DD:08<A9<D8:00E8,0000:
;
;		Subtract 0x0101 from count down to give current count
;
;		RC is measured by applying a Logic 1 to PA3 or PA2.
;		Count up RCcounter until PA4 gets to logic 1
;		When the voltage gets to logic one on PA4 
;
;		Rmin,Rnow,Rmax are counts. 
;
;		Convert the ratio of (Rnow-Rmin)/(Rmax-Rmin) to 0 to 5 min duration
;		using a successive approximation.
;		
;		Duration is in seconds.
;		Countdown-0x0101= seconds to go
;		* - timer running
;		

;

; AUTHOR:  	Douglas Rice
; Copyright 2001
;
;	Crystal:	32kHz
;
;	I/O used:-
;	PORTA:0	RS232 debug for Manufacture / Design Test 
;	PORTA:1 not used 
;	PORTA:2 RC measurement B 
;	PORTA:3 RC measurement A
;	PORTA:4	RC discharge
;
;	
;	PORTB:7	NC		
;	PORTB:6	NC
;	PORTB:5	IP: Trigger A
;	PORTB:4	IP: RC Timer
;	PORTB:3	Output D - RC Timer 
;	PORTB:2	Output C - While timing 2 Hz signal
;	PORTB:1	Output B - Goes low for 1 second at end
;	PORTB:0	Output A - Goes low for 1 second at start
;			Char input emulates button presses
;			If LSB is 1, emulate SET  pressed
;			If LSB is 0, emulate NEXT pressed
;
;
;	(Bi Colour LED) {set Button}
;			{NEXT Button }
;
;
;	On Trigger A start Timer.
;	Start A timer
;	Start 5 Min timer
;	Start C Osc
;
;	On Time out of 5 min timer
;	Start B timer
;
;
;	Main loop Start RC timer.
; 	 Set Output D
;	 test input RB4 If High Set PA2
;	When it goes High Calculate 5 min timer Value.
;
;
	INCLUDE "p16F84.inc"       
;	INCLUDE "p16C84.inc"       
;
;**************************************************************************

        LIST P=16F84, R=DEC
	__idlocs  0x1234
	__config  _XT_OSC&_PWRTE_ON & _WDT_OFF

	cblock 	0x0c
		; variables used for context saving 
		w_temp
		status_temp
	endc

;--------------------------------------------------------------------------
; Sec 0. Build Constants - uncomment to include in the code.
;--------------------------------------------------------------------------
;

;buildDEBUG	; if defined code easier to debug when single stepping.
;buildCircuit1
buildCircuit2
buildMacroRS232
;--------------------------------------------------------------------------
; Sec 1. Equates and Constants
;--------------------------------------------------------------------------


		; The General Purpose Registers start at the end of the 
		; Special Purpose Registers.


; IPMentState values
; in normal running or in time setting modes
;
SET_NORM	EQU	0
SET_SET	EQU	2

; STstate flag to indicate if beep is enabled.
; STstate is a variable of flag bits
STrelay		EQU	0
STbeep		EQU	1


; DoTimeSlice bits
; these bits are set to schedule a timer chain event
DTsS4event	EQU	0	; 
DTsSevent	EQU	1
DTsMevent	EQU	2
DTsHevent	EQU	3
DTtmrA		EQU	4
DTtmr5min	EQU	5
DTtmrB		EQU	6

;--------------------------------------------------------------------------
; Sec 1.1 Button and LED 
;--------------------------------------------------------------------------

OP_A		EQU	0	; start
OP_B		EQU	2	; stop
OP_C		EQU	1	; flash
OP_D		EQU	3

BUTTON_SET	EQU	4
BUTTON_TRIG	EQU	5
BUTTON_ONOFF	EQU	6
BUTTON_OPENED	EQU	7	; If change then record time

;--------------------------------------------------------------------------
; Sec 1.2 Button and LED 
;--------------------------------------------------------------------------

; PORTA binary values
; some port A bits are set in AWAI.inc


PORT_RS232	EQU	PORTA
RS232OUT	EQU	0

;--------------------------------------------------------------------------
; Sec 1.3 Table Macro
;--------------------------------------------------------------------------


TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary near " START   " "
	endif
	endm


;--------------------------------------------------------------------------
; Sec 2.0 Variables
;--------------------------------------------------------------------------
; Variables start 0x0C
;
; Variables for FlashLED ( FL ) process 
;

		;	DoTimeSlice Bits, to schedule, set bit
		;	0	run 	onQtrSecondTick
		;	1	run	onSecondTick
		;	2	run	onMinuteTick
		;	3	run	onHourTick
		;	4	run 	tmrA
		;	5	run 	tmr5Min
		;	6	run	TmrB
		;	7	


		cblock 	

DoTimeSlice
		
STstate	

CLtmr4sec	; counts quarter seconds

CLnbcdS 	; stores current time
CLnbcdM 
CLnbcdH 

CLdbcdM 	; stores the time being input
CLdbcdH 

CLbbcdM 	; stores the time being beeped out
CLbbcdH 
CLTNum		; Index into EEPROM when updating an ON/OFF time


IPlast		; Input variables
IPnew		
IPbuttonEvent	
IPsetState	

OPcnt		; Count 
OPopcnt		; Count 

RCcounter	; Resistor Capacitor measruement 
RCresultMin	; Min
RCresultMax	; Max
RCresultC	; 
RCresultA	; 
RCresultB	; 


RCresult	; 
RCresultSum	; 


SRout 		; for RS232 routines

SRin	; Used in an interupt routine for serial Input
SRcnt		
SRloopCnt

TM5minH		; 5 Min timer  High
TM5minL		; 5 Min timer  Low

BILEDPORT	; park Bi color led off PORTA.

PORTBtemp

	;
		ENDC
	ifdef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTBtemp
	endif
	ifndef 	buildDEBUG	; Build to aid debug
PORT_TIME	EQU	PORTB
	endif
;--------------------------------------------------------------------------
; Sec 3. Program Code
;--------------------------------------------------------------------------

                ORG     0       
                GOTO    Start

		ORG 	4
		GOTO 	Intrtn

;--------------------------------------------------------------------------
; Sec 3.1	Main Program Init Code
;--------------------------------------------------------------------------

Start

		ifdef buildDEBUG
		goto	MRcalcABdivC
		endif

		CLRF	INTCON
		; Set up PortB:<1:3> as outputs
		BSF	STATUS, RP0
		MOVLW	0XF0
		MOVWF	TRISB
		MOVWF	PORT_TIME

		MOVLW	0xE0	; Make PORTA 4:0 outputs
		MOVWF	TRISA
		BCF	STATUS, RP0

		; clear all pending events
		CLRF	DoTimeSlice	
		CALL	IPtimeslice
		CLRF	IPbuttonEvent
		DECF	IPbuttonEvent,F
	
		CLRF	IPsetState
		CLRF	CLdbcdH
		CLRF	CLdbcdM
		CLRF	CLbbcdH
		CLRF	CLbbcdM

		; Change Prescaler
		CLRWDT
		; Configure Tmr 0
		BSF	STATUS,RP0
		; Set up prescaller for 8192 / 16 = 512 ticks perseconds.
		MOVLW	1 << INTEDG | 0x0 + 2 	; For Tmr0 0=/2, 1=/4, 2=/8,3=/16
		MOVWF	OPTION_REG
		BCF	STATUS,RP0

		MOVLW	0x0 
		MOVWF	CLnbcdS
		MOVWF	CLnbcdH
		MOVWF	CLnbcdM

		; Calibrate A and B
		; Measure A or min
		
		; Measure B or max
		; Calc Range
		; 

		BSF	PORT_TIME,OP_A
		BSF	PORT_TIME,OP_B
		BCF	PORT_TIME,OP_C

		; Calibrate the Pot

		CALL	RCdischargeC
		CALL	RCdelay
		CALL	RCmeasure1
		MOVFW	RCresult
		MOVWF	RCresultMin
		MOVWF	RCresultA


		CALL	RCdischargeC
		CALL	RCdelay
		CALL	RCmeasure2
		MOVFW	RCresult
		MOVWF	RCresultMax
		MOVWF	RCresultB


		MOVLW	4
		MOVWF	CLtmr4sec
		
	; Make sure that the Serial OP does sometime.

		; Enable Interupts
		MOVLW	1 << GIE | 1 << INTE
		MOVWF	INTCON

		GOTO	MainLoop

;--------------------------------------------------------------------------
; Sec 3.2	Main Program 
;--------------------------------------------------------------------------

MainLoop	
	; Enable Interupts
	; Disable Interupts
MLdisableInt	
		CALL	CLtimeslice

		; Any events that exist
		BTFSC	DoTimeSlice,DTsS4event	; On 1/4 Second Tick
		CALL	SS4tick

		; Any events that exist
		BTFSC	DoTimeSlice,DTsSevent	; On Second Tick
		CALL	SStick

;		BTFSC	DoTimeSlice,DTsMevent	; On Minute Tick
;		CALL	SMtick

;		BTFSC	DoTimeSlice,DTsHevent		; On Hour Tick
;		CALL	SHtick

;		BTFSC	DoTimeSlice,DTSRwrtEvent	; Serial Display
;		CALL	SRwrtTime

;		BTFSS	IPbuttonEvent,BUTTON_SET	; On BUTTON_SET Press
;		CALL	IPsetPressed

		BTFSS	IPbuttonEvent,BUTTON_TRIG	; On BUTTON_TRIG Press
		CALL	IPnextPressed

;		BTFSS	IPbuttonEvent,BUTTON_ONOFF	; On BUTTON_ONOFF Press
;		CALL	IPonoffPressed

;		BTFSS	IPbuttonEvent,BUTTON_OPENED	; On BUTTON_OPENED Press
;		CALL	IPopenedPressed


		GOTO 	MainLoop
		

;--------------------------------------------------------------------------
; Sec 3.3 	Jump Table for running or time setting mode
;--------------------------------------------------------------------------


IPsetJump
	;
	; W=1 for SET, 
	; W=0 from NEXT
	;
IPjmpTableLcd4Start
	ADDWF	IPsetState,W
	CLRF	PCLATH		; Assume that the tabel is in the bottom 256 byte
	ANDLW	0x03
	ADDWF	PCL,f
	; set choice State Table - limited to 2 states and two events
	GOTO	STtoggle	; SET:OFF,NEXT
	GOTO	STtoggle	; SET:OFF,NEXT
	GOTO	STtoggle	; SET:OFF,NEXT
	GOTO	STtoggle	; SET:OFF,NEXT

	TEST_STRADDLE IPjmpTableLcd4Start


;--------------------------------------------------------------------------
; Sec 4.	Subroutines, procedures and functions
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Sec 4.1	Button Poll Routine
;--------------------------------------------------------------------------
		
IPtimeslice
		;	----___
		;	--_____
		;
		; This reads all Port B inputs and looks for Press
		MOVFW	IPnew
		MOVWF	IPlast
		MOVFW	PORT_TIME
		MOVWF	IPnew
		; IP last contains new setting, IPlast contains previous
		COMF	IPlast,W
		IORWF	IPnew,W
		; now force IPbuttonEvent bits to low for new pressed button
		; the service routine should set the bit to clear the event.
		ANDWF	IPbuttonEvent,F
		RETURN



;--------------------------------------------------------------------------
; Sec 4.3	Clock Chain Routine
;--------------------------------------------------------------------------
		
CLtimeslice

		; Assuming:
		; 	Crystal = 32.768 kHz
		;	Prescaller ticks at Crystal / 4 or 8192 ticks per second.
		;	Prescaller set to divide  so
		;	Tmr0 times out 4 times per second.
		;
	ifdef 	buildDEBUG	; Build to aid debug
		GOTO	CLtimeslice1
	endif
		BTFSS	INTCON,T0IF
		RETURN

CLtimeslice1

		; TMR0 timeout
		BCF	INTCON,T0IF
		BSF	DoTimeSlice,DTsS4event

		; This ticks 4 times per second, 
		; 
		DECFSZ	CLtmr4sec,f
		RETURN
		; divide by 4
		MOVLW	4
		MOVWF	CLtmr4sec
		GOTO	CLbcdS

;--------------------------------------------------------------------------
; Sec 4.4	BDC Clock Chain Routine
;--------------------------------------------------------------------------

		; BCD Counter	- Seconds
CLbcdS
		BSF	DoTimeSlice,DTsSevent
		INCF	CLnbcdS,f
		MOVF	CLnbcdS,w
		; binary add bcd digits to make 60 overflow digits.
		ADDLW	0xA6	; 10 + 6 = 16, overflows digit 6 for 10, 7 for 9, 8 for 8, 9 for 7, A for 6
		BC	CLbcdS1
		BDC	CLbcdS2
		RETURN

CLbcdS2		; unit counter carry to tens counter
		MOVFW	CLnbcdS
		; Add 1 to tens, subtract units to zero
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdS
		RETURN

CLbcdS1		
		CLRF	CLnbcdS
		; and on to the next counter

		; BCD Counter	- Minutes

CLbcdM
		BSF	DoTimeSlice,DTsMevent	
		INCF	CLnbcdM,f
		MOVF	CLnbcdM,w
		ADDLW	0xA6	; 10 + 6 = 16, overflows digit 6 for 10, 7 for 9, 8 for 8, 9 for 7, A for 6
		BC	CLbcdM1
		BDC	CLbcdM2
		RETURN

CLbcdM2		; unit counter carry to tens counter
		MOVFW	CLnbcdM
		; Add 1 to tens, subtract units to zero
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdM
		RETURN

CLbcdM1		
		CLRF	CLnbcdM
		; and on to the next counter

		; BCD Counter	Hours

CLbcdH
		BSF	DoTimeSlice,DTsHevent
		INCF	CLnbcdH,f
		MOVF	CLnbcdH,w
		ADDLW	0xA6		
		BC	CLbcdH1
		BDC	CLbcdH2
		; check for 24 hours
		MOVF	CLnbcdH,w
		ADDLW 0x00 - 0x24		
		SKPNZ	
		CLRF	CLnbcdH
		RETURN

CLbcdH2		; unit counter carry to tens counter
		MOVFW	CLnbcdH
		ADDLW	0x10 - 0x0A
		MOVWF	CLnbcdH
		RETURN

CLbcdH1		
		CLRF	CLnbcdH
		; and on to the next counter
		RETURN

;--------------------------------------------------------------------------
; Sec 4.5	Event Handler Routines - Timer
;--------------------------------------------------------------------------
SS4tick		;  do this 4 times a second
		BCF	DoTimeSlice,DTsS4event
		CALL	IPtimeslice

		BTFSS	DoTimeSlice,DTtmr5min
		GOTO 	SS4jmpTable1Start

		; TOGGLE OP C to  flash the Recording
		MOVLW	1 << OP_C
		XORWF	PORT_TIME,F

SS4jmpTable1Start
	; jumptample to do a different routine for each quarter second.
	MOVLW	high( $ )
	MOVWF	PCLATH		; Assume that the tabel is in the bottom 256 byte
	DECF	CLtmr4sec,W
	ANDLW	0x03
	ADDWF	PCL,f
	; set choice State Table - limited to 2 states and two events
	GOTO	SS4tickQ3
	GOTO	SS4tickQ2
	GOTO	SS4tickQ1
	GOTO	SS4tickQ0
	TEST_STRADDLE SS4jmpTable1Start


SS4tickQ0	;  do this in Q0 of second
		; do litte here as On second chain
		RETURN



SS4tickQ1	;  do this in Q1 of second
		; do litte here as On second chain
		; measure  RC1 pot A


		CALL	RCmeasure1
		MOVFW	RCresult
		MOVWF	RCresultA
		MOVWF	RCresultC
		;
		; Copy latest 0 to 5 min is 0 to 300 seconds
		;

		CALL	MRcalcABdivC
		; Calculated next delay

		RETURN



SS4tickQ2	;  do this in Q2 of second
		; Serial Debug 
		MOVLW	0x0D
		CALL	SRtxChar	
		MOVLW	0x0A
		CALL	SRtxChar	
		MOVLW	't'
		CALL	SRtxChar	

		SWAPF	MMmeas,W
		CALL	SRwrtHexNibble	
		MOVFW	MMmeas
		CALL	SRwrtHexNibble	
		MOVLW	':'
		CALL	SRtxChar	

	
		SWAPF	RCresultA,w
		CALL	SRwrtHexNibble	
		MOVFW	RCresultA
		CALL	SRwrtHexNibble	

		MOVLW	'+'
		CALL	SRtxChar	

		SWAPF	RCresultB,w
		CALL	SRwrtHexNibble	
		MOVFW	RCresultB
		CALL	SRwrtHexNibble	

		MOVLW	'='
		CALL	SRtxChar	
		
		MOVFW	RCresultA
		ADDWF	RCresultB,w
		MOVWF	RCresultSum

		SWAPF	RCresultSum,w
		CALL	SRwrtHexNibble	
		MOVFW	RCresultSum
		CALL	SRwrtHexNibble	


		MOVLW	':'
		CALL	SRtxChar	

		SWAPF	RCresultMin,W
		CALL	SRwrtHexNibble	
		MOVFW	RCresultMin
		CALL	SRwrtHexNibble	

		MOVLW	'<'
		CALL	SRtxChar	

		SWAPF	RCresultC,W
		CALL	SRwrtHexNibble	
		MOVFW	RCresultC
		CALL	SRwrtHexNibble	

		MOVLW	'<'
		CALL	SRtxChar	


		SWAPF	RCresultMax,W
		CALL	SRwrtHexNibble	
		MOVFW	RCresultMax
		CALL	SRwrtHexNibble	

		MOVLW	':'
		CALL	SRtxChar	

		SWAPF	MMtargetTh,w
		CALL	SRwrtHexNibble	
		MOVFW	MMtargetTh
		CALL	SRwrtHexNibble	

		SWAPF	MMtargetTl,w
		CALL	SRwrtHexNibble	
		MOVFW	MMtargetTl
		CALL	SRwrtHexNibble	
		MOVLW	','
		CALL	SRtxChar	

		SWAPF	TM5minH,w
		CALL	SRwrtHexNibble	
		MOVFW	TM5minH
		CALL	SRwrtHexNibble	

		SWAPF	TM5minL,w
		CALL	SRwrtHexNibble	
		MOVFW	TM5minL
		CALL	SRwrtHexNibble	

		MOVLW	':'
		CALL	SRtxChar	
		RETURN

SS4tickQ3	;  do this in Q3 of second
		; measure  RC2 pot B
		CALL	RCmeasure2
		MOVFW	RCresult
		MOVWF	RCresultB
		RETURN

SS4tickQ4	;  do this in Q4 of second

		RETURN

SStick		
		; this will occur one second 
		BCF	DoTimeSlice,DTsSevent

		; If the TMR A or B running clear output 
		; If TMR A primed Clear output
		BTFSS	DoTimeSlice,DTtmrA
		BSF	PORT_TIME,OP_A		
		; clear event
		BCF	DoTimeSlice,DTtmrA
		;

		; If TMR B primed Clear output
		BTFSS	DoTimeSlice,DTtmrB
		BSF	PORT_TIME,OP_B		
		; clear event
		BCF	DoTimeSlice,DTtmrB
		;

		BTFSS	DoTimeSlice,DTtmr5min
		RETURN


		MOVLW	'*'
		CALL	SRtxChar	


		DECFSZ	TM5minL,f
		RETURN
		DECFSZ	TM5minH,f
		RETURN

		MOVLW	'F'
		CALL	SRtxChar	
	
		;		
		; The 5 min timer has timed out stop the timers
		BSF	DoTimeSlice,DTtmrB
		BCF	DoTimeSlice,DTtmr5min
		; Make Output B low.
		BCF	PORT_TIME,OP_B		
		BCF	PORT_TIME,OP_C

		RETURN

SMtick		
		; minute ticks
		BCF	DoTimeSlice,DTsMevent
		RETURN

SHtick		; Turn Off On the Hour
		BCF	DoTimeSlice,DTsHevent
		RETURN

;--------------------------------------------------------------------------
; Sec 4.6	Event Handler Routine - Buttons
;--------------------------------------------------------------------------
; come here and combine the current state and the current event to index into 
; the state jump table.
;

IPsetPressed	
		BSF	IPbuttonEvent,BUTTON_SET
		RETURN

IPnextPressed	
		BSF	IPbuttonEvent,BUTTON_TRIG
		; if timer is running 
		BTFSC	DoTimeSlice,DTtmr5min
		RETURN

		; As button pressed, 
		; re time Second ticks to 1 second after Button going low
		; 
		; MOVLW	4 ; was 4 but seems to run for 2 seconds
		MOVLW	1

		MOVWF	CLtmr4sec
		
		; Set output A low and set TMR A
		BCF	PORT_TIME,OP_A		

		BSF	DoTimeSlice,DTtmrA
		BSF	DoTimeSlice,DTtmr5min


		INCF	MMtargetTh,w
		MOVWF	TM5minH

		INCF	MMtargetTl,w
		MOVWF	TM5minL


		BCF	PORT_TIME,OP_A
		RETURN

IPonoffPressed	
		BSF	IPbuttonEvent,BUTTON_ONOFF
		RETURN

IPopenedPressed	
		; 
		BSF	IPbuttonEvent,BUTTON_OPENED
		RETURN


;--------------------------------------------------------------------------
; Sec 4.7	Rotine to calc  A*B/C by sucessive approximation
;--------------------------------------------------------------------------

		
MRcalcABdivC
	; I need to calc a count between 0 and 5 minutes or 300 seconds
	; maxTime = 300 
	; I have calibrated the pot and measured min, max and now.
	; meas  = now-min
	; range = max-min
	;
	; I need to calc meas/range * max Time = count
	;

	cblock
MMmeas
MMrange
MMrangeT
MMref
MMtarget
MMrefT
MMtargetTh
MMtargetTl
MMcount
	endc


  ifdef  buildDEBUG
	; calc meas and range
	MOVLW	0x0a
	MOVWF	RCresultMin
	MOVLW	0xd0
	MOVWF	RCresultMax
	MOVLW	0x8B
	MOVWF	RCresult

	MOVLW	0x30
	MOVWF	RCresultA

	MOVLW	0xA5
	MOVWF	RCresultB


  endif

  ifdef buildCircuit1

	; w=f-w
        ; w=Max-Min
	; Calc Range
	MOVFW	RCresultMin
	SUBWF	RCresultMax,w
	MOVWF	MMrange

	; After calibrattion if current is less than A, then the timer is too long
	; so add code to prevent it going below 'rails'
		
	; Measure F-min
	; Calc Meas

	MOVFW	RCresultMin
	SUBWF	RCresultC,w
	MOVWF	MMmeas

	; If there is a carry we may have a problem
	BC MRcalcABdivC1
	; if there has been a carry then RCresult may be less than RCresultA
	; but if RCresult is > 128 then this is not a problem
	MOVFW	RCresultC
	ADDLW	0
	SKPC
	
	; if there has been a carry then RCresult < RCresultA
	CLRF	MMmeas
	GOTO	MRcalcABdivC1
  endif


  ifdef	buildCircuit2

	; This code should only be used with Circuit 2
	MOVFW	RCresultC
	MOVWF	MMmeas

	; Now calculate Range , A+B
	CLRC
	MOVFW	RCresultA
	ADDWF	RCresultB,w
	MOVWF	MMrange

	CLRF	RCresultMin

	MOVFW	MMrange
	MOVWF	RCresultMax

  endif

MRcalcABdivC1
        
	; set to 150, not 300 to save 16 arithmatic
	MOVLW	.150
	MOVWF	MMrangeT; this is divided by 2
	;
	CLRF	MMtarget
	CLRF	MMtargetTh
	CLRF	MMtargetTl
	movlw	8
	movwf	MMcount
MMcalcLoop	
	clrc
	rrf	MMrange,f
;	MMrangeT needs to be 300 seconds which is > 255 so divide by 2 now

	; calc  if MMmeas > MMtarget+MMrange/2	

	clrc
	movfw	MMtarget
	ADDWF	MMrange,w
	SUBWF	MMmeas,w	; w= f-w
	BNC	MMcalc 
	BZ	MMcalc 

	; work out new target for meas
	movfw	MMrange
	addwf	MMtarget,f
	; Slave Calc the Timer value
	movfw	MMrangeT
	addwf	MMtargetTl,f
	BNC	MMcalc 
	incfsz	MMtargetTh,f
MMcalc
	clrc
	rrf	MMrangeT,f

	decfsz	MMcount,f
	goto MMcalcLoop



	RETURN


;--------------------------------------------------------------------------
; Sec 4.7	Relay Control Routine
;--------------------------------------------------------------------------

STtoggle	RETLW	SET_NORM

STtoggleOn	RETLW	SET_NORM

STtoggleOff	RETLW	SET_NORM

STturnOn	RETURN


STturnOff	RETURN


;--------------------------------------------------------------------------
; Sec 4.8	EEprom Routines
;--------------------------------------------------------------------------

EEread		
		; w contains address
		BCF	STATUS,RP0
		MOVWF	EEADR
		BSF	STATUS,RP0
		BSF	EECON1,RD
		BCF	STATUS,RP0
		MOVFW	EEDATA
		RETURN

EEwrt		; blocking, EEADR and EEDATA set up.
		BCF	STATUS,RP0
		MOVWF	EEDATA
		BSF	STATUS,RP0
		BCF	INTCON,GIE
		BCF	EECON1,EEIF
		BSF	EECON1,WREN
		MOVLW	0x55
		MOVWF	EECON2
		MOVLW	0xAA
		MOVWF	EECON2
		BSF	EECON1,WR
		; Now wait for EEwrt to finish write
EEwaitForWrt	; block until EEPROM write has finished.
		BCF	EECON1,WREN	; disable EROM
		BTFSS	EECON1,EEIF
		GOTO	EEwaitForWrt
		BCF	EECON1,EEIF	; EEPROM finished
		BCF	STATUS,RP0
		BSF	INTCON,GIE

		RETURN
		

;--------------------------------------------------------------------------
; Sec 4.9	RC measurment routine
;--------------------------------------------------------------------------
;				

;    GND --| |--+---[ R2 ]---- RA2
;	    C 	|
;		+---[ R3 ]---- RA3
;		|
;		+---[ R4 ]---- RA4/TOCLKI
;
; To measure CR enable one RA4 as an output and an other as an
; input. Set RAOutput to start charging C, use the input to see if
; it has got to logic 1.
;
;
;
RCmeasure1
		; Set RA3 pin as output
		; Set output to high
		CLRF	RCcounter
		; Set up Port A<4> for Input.
		; Set up Port A<3> for Output
		BSF	STATUS, RP0
		BSF	TRISA,4
		BCF	TRISA,3
		BSF	TRISA,2
		BCF	STATUS, RP0
		; Set output low to discharge
		BCF	PORTA,3
;	RCmeas1DisLoop
;		DECFSZ	RCcounter
;		GOTO 	RCmeas1DisLoop
		
		; Set output high to start Measurement
		BSF	PORTA,3
RCmeas1Loop	
		BTFSC	PORTA,4
		GOTO	RCmeasure1End
		; don't end but leave Cont in W
		INCFSZ	RCcounter
		GOTO 	RCmeas1Loop
		; limit max
		DECFSZ	RCcounter

RCmeasure1End
		MOVFW	RCcounter
		MOVWF	RCresult
		BCF	PORTA,3
		GOTO	RCdischargeC

RCmeasure2
		; Set RA2 pin as output
		; Set output to high
		CLRF	RCcounter
		; Set up Port A<4> for Input.
		; Set up Port A<2> for Output
		BSF	STATUS, RP0
		BSF	TRISA,4
		BSF	TRISA,3
		BCF	TRISA,2
		BCF	STATUS, RP0
		; Set output low to discharge
		BCF	PORTA,2
;RCmeas2DisLoop
;		DECFSZ	RCcounter
;		GOTO 	RCmeas2DisLoop
		
		; Set output high to start Measurement
		BSF	PORTA,2
		; Set output high
		BSF	PORTA,2
RCmeas2Loop	
		BTFSC	PORTA,4
		GOTO	RCmeasure2End
		; don't end but leave Cont in W
		INCFSZ	RCcounter
		GOTO 	RCmeas2Loop
		DECFSZ	RCcounter

RCmeasure2End
		MOVFW	RCcounter
		MOVWF	RCresult
		BCF	PORTA,2

RCdischargeC
		; Set up Port A<4> for Output and PA2 and PA3 as inputs
		; Don't wait 
		BSF	STATUS, RP0
		BCF	TRISA,4
		BSF	TRISA,3
		BSF	TRISA,2
		BCF	STATUS, RP0
		; Set output low to discharge
		MOVLW 0
		MOVWF	PORTA

		RETURN


RCdelay
		MOVLW 0
		MOVWF	RCcounter
RCdelay1
		DECFSZ RCcounter,f
		goto RCdelay1
		RETURN


		
;--------------------------------------------------------------------------
; Sec 4.9	Serial Out
;--------------------------------------------------------------------------
;				
;		@ 8192 instructions a second, 
;		for 1200 baud, each bit = 6.82666 instructions
;		
;		---_01234567P---- 
;		-> Start     Stop
;		The output signal is inverted by the driver transistor
;
;		8192/	instrns	cumulative	% er
;		1200
;
;		0.000		0	
;	start	6.827	7	7	102.54%
;	0	13.653	7	14	102.54%
;	1	20.480	7	21	102.54%
;	2	27.307	6	27	98.88%
;	3	34.133	7	34	99.61%
;	4	40.960	7	41	100.10%
;	5	47.787	7	48	100.45%
;	6	54.613	7	55	100.71%
;	7	61.440	6	61	99.28%
;	8	68.267	7	68	99.61%
;	end	75.093	7	75	99.88%
;					

;#Is the output inverted? 
;# Normally the UART OP is inverted by the Line driver
;# but as design does not uses a driver invert the bits
RS232opHigh	MACRO	
		BCF	PORT_RS232,RS232OUT
	endm

RS232opLow	MACRO	
		BSF	PORT_RS232,RS232OUT
	endm

RS232opBit	MACRO   bit
		BTFSC	SRout,bit		; 1 if false, 2 if skip
		GOTO	SRtxCharA#v(bit)	; 	t+1 instructions
		NOP			;t+2
		;BSF	PORT_RS232,RS232OUT		;t+3
		RS232opLow
		GOTO	SRtxCharA#v(bit)a	;t+4
SRtxCharA#v(bit)	;BCF	PORT_RS232,RS232OUT		;	t+3
		RS232opHigh

		NOP			;	t+4
		NOP			;	t+5
SRtxCharA#v(bit)a	NOP			;t+6	t+6
	endm

	ifdef buildMacroRS232
SRtxChar
		;BSF	PORT_RS232,RS232OUT		;		t=0
		RS232opLow
;		MOVLW	'U'
		MOVWF	SRout

		NOP			;		t+1
		NOP			;		t+2		
;		NOP			;		t+3
		; These take 4 instruction cyles
		; t=0
		NOP


		RS232opBit	0
		RS232opBit	1
		RS232opBit	2
		RS232opBit	3
		RS232opBit	4
		RS232opBit	5
		RS232opBit	6
		RS232opBit	7
		; Stop Bit
		RS232opHigh
;		BCF	PORT_RS232,RS232OUT

		NOP
;		NOP
;		NOP
;		NOP
;		NOP
;		NOP
		RETURN
	endif

SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar

		

;--------------------------------------------------------------------------
; Sec 5.0	Interrupt Routines
;--------------------------------------------------------------------------

Intrtn	

		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		; Which Interupt ?
		; No Context Save in these routines.

IntrtnEnding
		BCF 	INTCON,INTF
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt


CodeEnd		goto	CodeEnd

;--------------------------------------------------------------------------
; Program End
;--------------------------------------------------------------------------




;--------------------------------------------------------------------------
; EEPROM data
;--------------------------------------------------------------------------

 
		ORG	0x2100


	cblock
LASTcont
	endc

               END


; Thoughts on measuring RC.
; The timer used above uses an RC. What is the accuracy that can be achieved.
; How important is this.

; Calibration.

To eliminate the temperature and component values, the code could measure 
each arm of the pot very easily.

Then to calibrate the unit, 
	set pot to start. 	measure A and B
	set pot to end.		measure A and B.

	then work out 

